"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuickVisualizeView = exports.QuickVisualizeModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const powerbi_client_1 = require("powerbi-client");
const version_1 = require("./version");
const utils_1 = require("./utils");
require("../css/report.css");
const quickCreateEmbedUrl = "https://app.powerbi.com/quickCreate";
const reportCreationMode = "QuickExplore" /* QuickExplore */;
const quickCreateTokenType = powerbi_client_1.models.TokenType.Aad;
class QuickVisualizeModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: QuickVisualizeModel.model_name, _model_module: QuickVisualizeModel.model_module, _model_module_version: QuickVisualizeModel.model_module_version, _view_name: QuickVisualizeModel.view_name, _view_module: QuickVisualizeModel.view_module, _view_module_version: QuickVisualizeModel.view_module_version, _embed_config: {}, _embedded: false, _token_expired: false, _init_error: null, container_height: 0, container_width: 0 });
    }
    ;
}
exports.QuickVisualizeModel = QuickVisualizeModel;
QuickVisualizeModel.model_name = 'QuickVisualizeModel';
QuickVisualizeModel.model_module = version_1.MODULE_NAME;
QuickVisualizeModel.model_module_version = version_1.MODULE_VERSION;
QuickVisualizeModel.view_name = 'QuickVisualizeView';
QuickVisualizeModel.view_module = version_1.MODULE_NAME;
QuickVisualizeModel.view_module_version = version_1.MODULE_VERSION;
class QuickVisualizeView extends base_1.DOMWidgetView {
    render() {
        const newDivElement = document.createElement('div');
        newDivElement.style.visibility = 'hidden';
        this.el.appendChild(newDivElement);
        this.quickCreateContainer = newDivElement;
        this.embedConfigChanged();
        // Observe changes in the traitlets in Python, and define custom callback.
        this.model.on('change:_embed_config', this.embedConfigChanged, this);
        this.model.on('change:container_height', this.containerSizeChanged, this);
        this.model.on('change:container_width', this.containerSizeChanged, this);
    }
    containerSizeChanged() {
        if (this.quickCreateContainer) {
            this.quickCreateContainer.style.height = `${this.model.get('container_height')}px`;
            this.quickCreateContainer.style.width = `${this.model.get('container_width')}px`;
        }
    }
    setTokenExpiredFlag() {
        this.model.set('_token_expired', true);
        this.touch();
    }
    embedConfigChanged() {
        const embedConfig = this.model.get('_embed_config');
        let quickCreateConfig = embedConfig;
        if (!quickCreateConfig) {
            this.model.set('_init_error', 'Embed configuration is missing');
            this.touch();
            return;
        }
        // Populate fixed configuration
        quickCreateConfig.type = 'quickCreate';
        quickCreateConfig.tokenType = quickCreateTokenType;
        quickCreateConfig.embedUrl = quickCreateEmbedUrl;
        quickCreateConfig.reportCreationMode = reportCreationMode;
        if (this.quickCreate) {
            // To avoid re-embedding Quick Create
            if (this.quickCreate.config.accessToken !== quickCreateConfig.accessToken) {
                // Set new access token
                this.quickCreate.setAccessToken(quickCreateConfig.accessToken);
                if (quickCreateConfig.accessToken) {
                    // Set token expiration listener to update the token TOKEN_REFRESH_THRESHOLD minutes before expiration
                    utils_1.setTokenExpirationListener(quickCreateConfig.accessToken, this);
                }
            }
            this.model.set('_embedded', true);
            this.touch();
            return;
        }
        else {
            // Refresh access token before embedding if token is expired
            if (!quickCreateConfig.accessToken || utils_1.getTokenExpirationTimeout(quickCreateConfig.accessToken) <= 0) {
                this.setTokenExpiredFlag();
                return;
            }
        }
        this.quickCreate = utils_1.powerbi.quickCreate(this.quickCreateContainer, quickCreateConfig);
        // TODO: show iframe when report is loaded once "loaded" event is implemented
        try {
            // this.el is updated with correct width when report is loaded. Using timeout until "loaded" event is implemented
            setTimeout(() => {
                if (quickCreateConfig.accessToken) {
                    // Set token expiration listener to update the token TOKEN_REFRESH_THRESHOLD minutes before expiration
                    utils_1.setTokenExpirationListener(embedConfig.accessToken, this);
                }
                // Set default aspect ratio
                const aspectRatio = 9 / 16;
                let width = this.model.get('container_width');
                let height = this.model.get('container_height');
                if (!width || !height) {
                    // Get dimensions of output cell
                    const DOMRect = this.el.getBoundingClientRect();
                    width = DOMRect.width || 980;
                    height = width * aspectRatio;
                }
                // Set dimensions of quick create container
                this.quickCreateContainer.style.width = `${width}px`;
                this.quickCreateContainer.style.height = `${height}px`;
                this.quickCreateContainer.style.visibility = 'visible';
            }, 500);
        }
        catch (error) {
            console.error(error);
        }
        // Notify that report embedding has started
        this.model.set('_embedded', true);
        // Commit model changes
        this.touch();
    }
}
exports.QuickVisualizeView = QuickVisualizeView;
//# sourceMappingURL=quickVisualize.js.map